//Simple graph class for plotting time series data

class dataGraph {
  PFont lFont;
  int gleft, gtop, gwidth, gheight, bc;
  float sc;
  int indata;
  int istart;
  int bcol;
  int zeroAt;
  boolean bkgndVisible;

  dataGraph (int gl, int gt, int gw, int gh, int bc) {
    gleft = gl;
    gtop = gt;
    gwidth = gw;
    gheight = gh;
    bcol = bc;
    lFont = createFont("Arial", 12);
    bkgndVisible = false;
  }

  int getGwidth() {
    return gwidth;
  }

  int getGheight() {
    return gheight;
  }

  void grBackground(int[] data, int indata, int instart, int col) {
    //    gleft = gl;
    //    gtop = gt;
    //    gwidth = gw;
    //    gheight = gh;
    //    bcol = bc;
    grBackground();
    
    float xincr = gwidth/float(indata);
    int ccol = col;
        pushStyle();
    pushMatrix();
    translate(gleft, gtop);
    stroke (ccol);
    for (int i = instart; i<indata+instart; i++) {
      if (data[i] == 1) {
        int xval = round(xincr*(i-instart));
        line(xval, 0, xval, gheight);
      }
    }
    popMatrix();
    popStyle();
    bkgndVisible = true;
  }


void grBackground(){
    pushStyle();
    pushMatrix();
    translate(gleft, gtop);
    fill (bcol);
    rect(0, 0, gwidth, gheight);
    popMatrix();
    popStyle();
    bkgndVisible = true;
  }

  void grTitle(String lab, int lSize, int lx, int ly, int col) {       //Title for whole graph
    grTitle(lab, lSize, lx, ly, col, LEFT);
  }

  void grTitle(String lab, int lSize, int lx, int ly, int col, int tAlign) {       //Title for whole graph
    pushStyle();
    pushMatrix();
    translate(gleft, gtop);
    fill(col);
    textAlign(tAlign);
    textSize(lSize);
    text (lab, lx, ly);
    popMatrix();
    popStyle();
  }

  void grResults(int col, String lab, int colOrd, int xOffset) {
     pushMatrix();
    translate(gleft, gtop);
    pushStyle();
    stroke(col);
    fill(col);
    textSize(12);
    int lh =  gheight+15*colOrd;
    text(lab, xOffset, lh);
    popMatrix();
    popStyle();
 }
  
  void grLabel(int col, String lab, int colOrd) {
    pushMatrix();
    translate(gleft, gtop);
    pushStyle();
    stroke(col);
    fill(col);
    int lh =  gheight+15*colOrd;
    line(0, lh, 50, lh);
    textSize(12);
    text(lab, 55, lh);
    popMatrix();
    popStyle();
  }

  void grxDivisions(int gminor, int gmajor, String lab, boolean zeroline) {           
    //put short lines at every gminorth interval, long lines every gmajorth minor interval
 grxDivisions(gminor, gmajor, lab, zeroline, gwidth);
  }
  
    void grxDivisions(int gminor, int gmajor, String lab, boolean zeroline, int gframes) {           
    //put short lines at every gminorth interval, long lines every gmajorth minor interval
    int index;
    pushMatrix();
    translate(gleft, gtop);
    pushStyle();
    if (zeroline == true) {
      stroke(128);
      line(0, gheight/2, gwidth, gheight/2);   //ZeroLine for data
    }
    fill(255);
    stroke(255);
    float gunit = float(gwidth)/float(gframes);
    for (int i = 0; i < gframes; i++) {
      index = round(i*gunit);
      if ((i%gminor) == 0) {
        line(index, gheight, index, round(19*gheight/20));
        line(index, 0, index, round(gheight/20));
        if ((i%(gminor*gmajor)) == 0) {
          line(index, gheight, index, round(9*gheight/10));
          line(index, 0, index, round(gheight/10));
        }
      }
    }
    textSize(16);
    text(lab, 10, round(9*gheight/10));
    popMatrix();
    popStyle();
  }

  void gryDivisions(int gminor, int gmajor, String lab) {           
       gryDivisions(gminor, gmajor, lab, false, -20, #EEEEEE);
  }
  
  void gryDivisions(int gminor, int gmajor, String lab, boolean zeroline, int yOffset, int col) {           
    //put short lines at every gminorth interval, long lines every gmajorth minor interval
    int index;
    pushMatrix();
    translate(gleft, gtop);
    pushStyle();
    if (zeroline == true) {
      stroke(128);
      line(0, gheight/2, gwidth, gheight/2);   //ZeroLine for data
    }
    fill(col);
    stroke(col);
    for (int i = 0; i < gheight; i++) {
      if ((i%gminor) == 0) {
        line(0, i, 10, i );
        line(gwidth, i, gwidth - 10, i);
        if ((i%(gminor*gmajor)) == 0) {
          line(0, i, 20, i);
          line(gwidth, i, gwidth-20, i);
        }
      }
    }
    textSize(16);
    textAlign(CENTER, CENTER);
    translate(yOffset,round(gheight/2));
    rotate(-HALF_PI);
    text(lab,0,0); 
    popMatrix();
    popStyle();
  }

  void graphData(float [] theData, int ndata, int startpos, int col, float scale) {
    graphData(theData, ndata, startpos, col, scale, 0, 0);
  }

  void graphData(float [] theData, int ndata, int startpos, int col, float scale, float offSet) {
    graphData(theData, ndata, startpos, col, scale, offSet, 0);
  }

  void graphData(float [] theData, int ndata, int startpos, int col, float scale, float offSet, int zeroAt) {
    if (bkgndVisible == false) {
      grBackground();
    }
    sc = scale;
    indata = ndata;
    istart = startpos;    
    pushMatrix();
    translate(gleft, gtop + (gheight-zeroAt));
    pushStyle();
    stroke(col);
    float xIncr = gwidth/indata;
    float xloc;
    float yloc;
    float lastXloc = 0;
    float lastYloc = 0;
    lastYloc = -(theData[istart]-offSet)*sc;
    for (int i = istart+1; i < istart+indata; i++) {
      xloc = lastXloc + xIncr;
      yloc = -(theData[i]-offSet)*sc;
      line(round(lastXloc), round(lastYloc), round(xloc), round(yloc));
      lastXloc = xloc;
      lastYloc = yloc;
    }
    popMatrix();
    popStyle();
  }

  void graphData( int [] theData, int ndata, int startpos, int col, float scale) {
    graphData (theData, ndata, startpos, col, scale, 0, 0);
  }

  void graphData( int [] theData, int ndata, int startpos, int col, float scale, float offset) {
    graphData (theData, ndata, startpos, col, scale, 0, zeroAt);
  }

  void graphData ( int [] theData, int ndata, int startpos, int col, float scale, float offSet, int zeroAt) {
    if (bkgndVisible == false) {
      grBackground();
    }
    sc = scale;
    indata = ndata;
    istart = startpos;    
    pushMatrix();
    translate(gleft, gtop + (gheight-zeroAt));
    pushStyle();
    stroke(col);
    float xIncr = gwidth*1.0/indata;
    float xloc;
    float yloc;
    float lastXloc = 0;
    float lastYloc = 0;
    lastYloc = -(theData[istart]-offSet)*sc;
    for (int i = istart+1; i < istart+ndata; i++) {
      xloc = lastXloc + xIncr;
      yloc = -(theData[i]-offSet)*sc;
      line(round(lastXloc), round(lastYloc), round(xloc), round(yloc));
      lastXloc = xloc;
      lastYloc = yloc;
    }
    popMatrix();
    popStyle();
  }

}

