// Construct synthetic noise by adding component elements
/*
Types: Smooth: summed sinusoids with a defined spectral shape
 Ramp: steady rise or fall intermingled with fixed plateaus
 Variable Triangle: varying change rate, but constant between poisson-distributed change points
 Random Steps: Step times with a Poisson function with defined statistics 
 Triangle: Constant velocity with changes in direction at Poisson intervals
 Perlin: Perlin noise
 All types are normalized to return range = 1
 
 */


class Vnoise {
  int samples, iKind;
  String kind;
  float dbOct, dbS;
  float []sf, sfBase;
  float sum, ssq, mean, rms;
  float sq2 = sqrt(2.0);
  int lcycles, hcycles, nWaves;          // bounds on the noise band
  int baseCycle = 4096;

  Vnoise (int s) {     //default consructor Sinusoid noise 6 db/octave
    lcycles = 1;
    hcycles = 32;
    dbOct = 6;
    kind = "Smooth";
    setupVnoise(s, kind, dbOct);
  }

  Vnoise (int s, String k, float db) {     //Low Pass noise
    lcycles = 1;
    hcycles = 32;
    setupVnoise(s, k, db);
  }

  Vnoise (int s, String k, float db, int lcy) {   //BandPass noise
lcycles = lcy;
setupVnoise(s, k, db);
  }

  void setupVnoise(int s, String k, float db) {
    samples = s;
    float noff, nmove;
    sfBase = new float[samples];
    sf = new float[samples];
    kind = k;
    nWaves = 0;
    dbOct = 12-2*db;
    if (k == "Smooth") {
      iKind = 0;
      float angleIncr;
      angleIncr = TWO_PI/baseCycle;
      for (int i = 0 ; i < baseCycle; i++) {
        sfBase[i] = sin(i*angleIncr % TWO_PI);
      }
      dbS = 0.0-dbOct/6;
      // Now we add random phase sine components as requested
      for (int j = lcycles; j <= hcycles; j++) {
        float jmult = pow(j, dbS);
        int phase = round(random(samples));
        for (int i = 0; i < samples; i++) {
          int index = (i*j + phase) % samples;
          sf[i] += sfBase[index]*jmult;           //sf is the noise array, unscaled
        }
      }
    }
    if (k == "Ramp") {
      iKind = 1;
      float sp;
      sf[0] = 0;
      sp = db+1;
      float sVel = .006;
      float sgVel = 1;
      float perFrame = .005*(sp + 0.2*sp*db);
      for (int i = 1; i < samples; i++) {
        if (random(1) < perFrame) {
          sgVel = floor(random(3))-1;
          sVel = random(.005*(sp + 0.2*sp*db*db));
        }

        sf[i] = sf[i-1] + sgVel*sVel;
      }
    }

    if (k == "VariableTriangle") {
      iKind = 2;
      float sp;
      sf[0] = 0;
      sp = db+1;
      float sVel = .015;
      float sgVel = 1;
      float perFrame = .003*(sp + 0.2*sp*db);
      for (int i = 1; i < samples; i++) {
        if (random(1) < perFrame) {
          sgVel *= -1;
          sVel = random(.005*(sp + 0.2*sp*db));
        }
        sf[i] = sf[i-1] + sgVel*sVel;
      }
    }

    if (kind == "RandomStep") {
      iKind = 3;
      float sp;
      sf[0] = 0;
      sp = db+1;
      //      println("db/Oct = " + dbOct);
      float perFrame = .003*(sp + 0.2*sp*db);
      for (int i = 1; i < samples; i++) {
        if (random(1) < perFrame) {
          sf[i] += round(gRandom(sp));
        } 
        else {
          sf[i] = sf[i-1];
        }
      }
    } 

    if (kind == "Triangle") {
      float sp;
      iKind = 4;
      sf[0] = 0;
      sp = db+1;
      float sVel = .01;
      float sgVel = 1;
      float perFrame = .003*(sp + 0.2*sp*db);
      for (int i = 1; i < samples; i++) {
        if (random(1) < perFrame) {
          sgVel *= -1;
        }
        sf[i] = sf[i-1] + sgVel*sVel;
      }
    }

    if (kind == "Perlin") {
      iKind = 5;
      noff = millis();
      nmove = .0002*(1.0 + db + 0.3*db*db);
      //      nmove = .002;
             println ("db = "+db+ ", nmove = "+nmove);
      for (int i = 0; i < samples; i++) {
        noff += nmove;
        sf[i] = noise(noff);
      }
    }

    if (kind == "SmallStep" ) {
      iKind = 6;
      sf[0] = 0;
      for (int i = 1; i < samples; i++) {
        if ((i % round(300/(db+1))) == 0) {
          sf[i] = sf[i-1] + gRandom(20);
        } 
        else {
          sf[i] = sf[i-1];
        }
      }
    }
    
    if (kind == "White") {         //Additive sine waves of Gaussian power and random phase

      iKind = 7;
      float wdb = db-1;
      nWaves = round(db + wdb*db/2 + 0.3);   //If db ranges from 1 to 5, nWaves ranges from 1 to 15
      hcy = lcycles+nWaves;
     float[] sinWave = new float[samples];
      //First create a lookup table for a sine wave of length 4096
      //Base wavelength is always 8192 samples no matter how many noise samples we want
      //If we want a finer resolution, multiply wBase and frame rate by a power of 2 
      int wBase = 4096;
     float inc = TWO_PI/wBase;
      for (int i = 0; i < wBase; i++) {
        sinWave[i] = sin(inc*i);
      }
      for(int i = 0; i< samples; i++) {
        sf[i] = 0;
      }
      for(int j = lcycles; j < hcy; j++) {
        int phase = round(random(wBase));
        float amp = sqrt(abs(randomGaussian()));
        for(int i = 0; i < samples; i++) {
          sf[i] += amp*sinWave[((phase+i)*j)%wBase];
          if( i == 1000) {
          }
        }
      }        
    }


    // Normalize the noise to have range -1 to +1
    float smq;
    float sfMax = -10000000;
    float sfMin = 10000000;
    float sfRange, sfMid;
    sum = 0;
    ssq = 0;
    mean = 0;

    for (int i = 0; i < samples; i++) {
      sum += sf[i];
      sfMax = max(sf[i], sfMax);
      sfMin = min(sf[i], sfMin);
    }
    sfRange = sfMax-sfMin;
    sfMid = (sfMax+sfMin)/2;
//    mean = sum/samples;
//    for (int i = 0; i< samples; i++) {
//      smq = sf[i] -mean;
//      ssq += smq*smq;
//    }
//    rms = sqrt(ssq/samples);
//    sfMax = -1000;
//    sfMin = 1000;
//    for (int i = 0; i < samples; i++) {  //Check that it is correct
//      sum += sf[i];
//      sfMax = max(sf[i], sfMax);
//      sfMin = min(sf[i], sfMin);
//    }
//    sfRange = sfMax-sfMin;
//    sfMid = (sfMax+sfMin)/2;
    for (int i = 0; i < samples; i++) {
      sf[i] = sf[i]-sfMid;             // centre around zero
      sf[i] = sf[i]*2/sfRange;           // make range -1 to +1
    }
  }   


  float getVnoise(int i) {   // collect sample "i" of the made noise
    return sf[i];
  }

  float [] getVnoise() {  // get the entire noise array
    return sf;
  }
  
  int getnWaves() {
    return nWaves;
  }

  float gRandom (float s) {   //return an approximately Gaussian random number with s.d. = s
    float gSum;
    gSum = 0;
    for (int i = 0; i < 16; i++) {
      gSum += random(s)-s/2;
    }
    gSum /= 4;
    return gSum;
  }
}

